#define DEBUG 1;

#ifndef GetOptions_h
#define GetOptions_h

#ifdef DEBUG
#include "Debug.h"
#endif

#include <string>
#include <map>
#include <sstream>
#include "Option.h"

using namespace option;
using namespace std;
struct Function;

class GetOptions
{
#ifdef DEBUG
    Debug dbg;
#endif
protected:
    typedef map<string, Option*>	optionMapType;
    typedef map<string, string>		stringMapType;
    typedef map<string, Function*>	functionMapType;
    optionMapType					optionMap;
    stringMapType					optionToIdMap;
    functionMapType					functionMap;
    stringMapType					callToFuncIdMap;
public:
    /*
    //	Default Constructor
    */
    GetOptions();

    /*
    //	addOption
    //		adds a command line option to this command line option interpreter
    //		-inputs - id - the identifier handle for this option
    //				- sOp - the short form of the option
    //				- lOp - the long form of the option
    //				- tp - the type of value that this option uses
    //		-return - 0 if it was successful
    */
    int addOption(const char* id, const char* sOp, const char* lOp, type_id tp);

    /*
    //	addFunction
    //		adds a command line function to this interpreter
    //		-inputs - id - the identifier of the funciton
    //				- callN - the call name of the function to be used in the command line to call the function
    //		-reutrn - 0 if successful
    //				- 1 if there is already a function with this id
    //				- 2 if there is already a function with this call name
    */
    int addFunction(const char* id, const char* callN);

    /*
    //	addFunction
    //		adds a command line function to this interpreter
    //		-inputs - id - the identifier of the funciton
    //				- callN - the call name of the function to be used in the command line to call the function
    //				- getopts - the GetOptions class to set to this function
    //		-return - 0 if successful
    //				- 1 if there is already a function with this id
    //				- 2 if there is already a function with this call name
    */
    int addFunction(const char* id, const char* callN, GetOptions* getopts);

    /*
    //	isFunctionActive
    //		checks if the function identified by func is active, i.e. the user entered it in command line
    //		-inputs - func - id of the function to check
    //		-return - true if the function is active, false otherwise
    */
    bool isFunctionActive(const char* func);

    /*
    //	setOptsToFunction
    //		sets the GetOptions object associated with this function
    //		-inputs - func - id of the function
    //				- getopts - the GetOptions object which is set to this function
    //		-return - 0 if successful
    //				- 1 if the function doesn't exist
    */
    int setOptsToFunction(const char* func, GetOptions* getopts);

    /*
    //	getFuncOpts
    //		get the GetOptions object of the function
    //		-inputs - func - the identifier for the function to retrieve
    //		-return - a pointer to the GetOptions object of the function,
    ///				Or NULL if the funciton doesn't exist
    */
    GetOptions* getFuncOpts(const char* func);

    /*
    //	parse
    //		parses the string for any and all options, placing the values in the corresponding options in the map
    //		-inputs - str - the string of options to be parsed
    //		-return - 0 if it successfully identified and parsed the whole string for options
    //				- a postive number indicating the number of options that were unidentifiable
    //				- a negative number indicating that no valid options could be found
    */
    int parse(const char* str);

    /*
    //	_type_Value
    //		returns the value of the option "id" assuming that the function type corresponds to the option's value type
    //		-inputs - id - the identifier of the option to get the value from
    //		-return - the value of the option id, assuming the function type corresponds to the option value type
    */
    int intValue(const char* id);
    const char* cStrValue(const char* id);
    double doubleValue(const char* id);
    bool boolValue(const char* id);

protected:
    /*
    //	testValidOptionValue
    //		tests whether the value string contains a valid value for this value type
    //		-inputs - val - the string containing the value to be tested
    //				- type - the value type to be tested for
    //		-return - true if the value is valid for this value type, false otherwise
    */
    bool testValidOptionValue(string val, type_id type);

    /*
    //	setValue
    //		sets the value of an the option
    //		-inputs - opt - the string taken from the command line which identifies the option
    //				- val - the value to be set to the option
    //		-return - an integer indicating success or not, 0 means successful
    */
    int setValue(const char* opt, const char* val);

    /*
    //	checkValidOp
    //		checks whether the string refers to a valid option
    //		-inputs - opt - the string, which will be tested
    //		-return - true if it's a valid option, false otherwise
    */
    bool checkValidOp(const char* opt);

    /*
    //	getType
    //		retrieves the value type specified for an option
    //		-inputs - opt - the option string, that identifies the option
    //		-return - the value type of the option or 0 if it isn't a valid option string
    */
    type_id getType(const char* opt);

    /*
    //	isValidPathName
    //		tests if the string is a valid path in this operating system
    //		-inputs - path - the string to be validated as a path
    //		-return - true if the string is a valid path, false otherwise
    */
    bool isValidPathName(string path);

    /*
    //	isFunc
    //		checks if func is an id for a function that has been added to 
    //		-inputs - func - identifier of the function
    //		-return - true if the function exists, false otherwise
    */
    bool isFunc(const char* func);

    /*
    //	isOp
    //		checks if id is an identifier for an option that already exists
    //		-inputs - id - identifier of the option
    //		-return - true if the option exists, false otherwise
    */
    bool isOp(const char* id);

    /*
    //	isCallName
    //		checks whether the string refers to a call name for a function that exists
    //		-inputs - opt - the string, which will be tested
    //		-return - true if it's a call name, false otherwise
    */
    bool isCallName(const char* callN);

    /*
    //	isOpActive
    //		checks if the option identified by op is active, i.e. the user entered it in command line
    //		-inputs - op - id of the option to check
    //		-return - true if the function is active, false otherwise
    */
    bool isOpActive(const char* op);

#ifdef DEBUG
public:
    void debugOutAllOptions()
    {
        dbg << "Options Dump...." << endl;
        map<string, Option*>::iterator iter = optionMap.begin();
        while (iter != optionMap.end())
        {
            dbg << "\t" << iter->second->identifier << endl;
            dbg << "\t\tShort: " << iter->second->shortOpt << endl;
            dbg << "\t\tLong: " << iter->second->longOpt << endl;
            dbg << "\t\tType: " << iter->second->type << endl;
            dbg << "\t\tValue: " << iter->second->value.c_str() << endl;
            iter++;
        }
        dbg << "End Dump" << endl;
    };

    Debug* getDebug()
    {
        return &dbg;
    };
#endif
};

struct Function
{
    const char*		identifier;
    const char*		callName;
    bool			isSet;
    GetOptions*		getopts;
};

#endif