
#ifndef FILEIO_H
#define FILEIO_H
#include <stdio.h>
#include <string.h>
#include "dbh.h"
#include <iostream>
#include <fstream>

/*
//	readHeader:
//		Reads Analyze header files for certain data
//		-inputs - filename - filename of the header file
//		-return	- a pointer to the read header or NULL if it could not be read
*/
dsr* readHeader(const char* filename);

/*
//	readHeader:
//		Reads Analyze header files for certain data
//		-inputs - filename - filename of the header file
//				- dims - pointer to the array that the dimensions will be returned in
//				- newFilename - pointer to the char array that will hold the filename of the
//								image file
//				- type - the datatype of the data
//		-return	- an integer which holds the number of dimensions of the data set
*/
int readHeader(const char* filename, int* dims, int* type, char* newFilename);

/*
//	readHeader:
//		Reads Analyze header files for certain data
//		-inputs - filename - filename of the header file
//				- dims - pointer to the array that the dimensions will be returned in
//				- newFilename - pointer to the char array that will hold the filename of the
//								image file
//				- type - the datatype of the data
//				- FOV - the field of view of the data
//		-return	- an integer which holds the number of dimensions of the data set
*/
int readHeader(const char* filename, int* dims, int* type, char* newFilename, double* FOV);

/*
//	getImageName:
//		Changes an Analyze header filename to an image filename
//		-inputs - filename - filename to have its extension changed
//		-return - a character string with the .img extension
*/
char* getImageName(const char* filename);

/*
//	getHeaderName:
//		Changes an Analyze image filename to an header filename
//		-inputs - filename - filename to have its extension changed
//		-return - a character string with the .hdr extension
*/
char* getHeaderName(const char* filename);

/*
//	swapHeaderBytes:
//		Swaps the byte order of a header file
//		-inputs - header - header structure to have bytes swapped
//		-return - nothing...changes header
*/
void swapHeaderBytes(dsr* header);

/*
//	readFile
//		reads an image file
//		-inputs - filename - filename to be read
//				- nDim - the number of dimensions
//				- dims - the dimensions
//				- type - the type of data
//		-return - returns a pointer to the data
*/
double* readFile(const char* filename, int nDim, int* dims, int type, bool bigEnd);

/*
//	readFile
//		reads an image file
//		-inputs - filename - filename to be read
//				- header - header of the image file
//		-return - returns a pointer to the data
*/
double* readFile(const char* filename, dsr* header);

/*
//	getSize
//		returns the size of data type
//		-inputs - type - the data type
//		-return - the size of the data type
*/
int getSize(int type);

void swapBytes(short int* num);

void swapBytes(int* num);

void swapBytes(float* num);

void swapBytes(double* num);

/*
//	writeHeader
//		writes a header file for an Analyze image
//		-inputs - filename - name of the header file to be written
//				- nDim - number of dimensions
//				- dim - the dimensions of the data
//				- voxDim - the physical size dimensions
//		-return - integer indicating successful writing or not
*/
int writeHeader(const char* filename, int nDim, int* dim, double* voxDim);

/*
//	writeHeader
//		writes a header file for an Analyze image
//		-inputs - filename - name of the header file to be written
//				- header - the header to be written to file
//		-return - integer indicating successful writing or not
*/
int writeHeader(const char* filename, dsr* header);

/*
//	writeImageFile
//		writes a Analyze image file
//		-inputs - filename - name of the image file to be written
//				- header - the header of the image file to be written
//				- data - the data to be written
//		-return - integer indicating successful writing or not
*/
int writeImageFile(const char* filename, dsr* header, double* data);

/*
//	initHeader
//		creates a default header
//		-return - pointer to an initialized, default header
*/
dsr* initHeader();

/*
//	readCalibFile
//		reads a calibration file
//		-inputs - filename - name of the calibration file to be read
//				- size - size of the calibration file to be read
//		-return - pointer to the calibration data
*/
double* readCalibFile(const char* filename, int size);

/*
//	writeCalibFile
//		writes a calibration file
//		-inputs - filename - name of the calibration file to be written
//				- size - size of the calibration file to be written
//				- data -pointer to the calibration data
//		-return - integer indicating success or not
*/
int writeCalibFile(const char* filename, int size, double* data);

/*
//	validateHeaderFile
//		checks the file to test if it is a valid Analyze header
//		-inputs - filename - name of the file to be validated
//		-return - a boolean value - true if the file is a valid header file, false otherwise
*/
bool validateHeaderFile(const char* filename);

/*
//	checkExtension
//		checks if the filename has the correct extension
//		-inputs	- filename - the filename to be checked
//				- ext - the string containing the extension(s) to check for
//					-format- *.ext *.nextextension *.next
//		-return - true if the filename has the extension, false otherwise
*/
bool checkExtension(const char* filename, const char* extension);

#endif